﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace TeachingLinq
{
    public interface IHomework
    {
        /// <summary>
        /// Count the number of words in the input collection (words are separated by wordDelimiters).
        /// <param name="lines">A collection of strings, containing each multiple words.</param>
        /// </summary>
        /// <returns>The number of words in the input collection.</returns>
        int Problem1(IEnumerable<string> lines);

        /// <summary>
        /// Count the number of unique (distinct) words in the input collection.
        /// <param name="lines">A collection of strings, containing each multiple words.</param>
        /// <returns>The number of distinct (unique) words in the input collection.</returns>
        /// </summary>
        int Problem2(IEnumerable<string> lines);

        /// <summary>
        /// Find the most frequent 10 words in the input collection, and their counts.
        /// <param name="lines">A collection of strings, containing each multiple words.</param>
        /// </summary>
        IEnumerable<Tuple<string, int>> Problem3(IEnumerable<string> lines);

        /// <summary>
        /// From each line of text extract just the vowels ('a', 'e', 'i', 'o', 'u').
        /// Example:
        /// Input: "How all occasions do inform against me",
        /// Output: Item1 => "How all occasions do inform against me",
        ///         Item2 => {o,a,o,a,i,o,o,i,o,a,a,i,e}
        /// <param name="lines">A collection of strings, containing each multiple words.</param>
        /// <returns>A list of lines and for each a list of vowels.</returns>
        /// </summary>
        IEnumerable<Tuple<string, IEnumerable<char>>> Problem4(IEnumerable<string> lines);

        /// <summary>
        /// Given a list of words find the number of occurrences of each of them in the input collection.
        /// <returns>For each word the count.  (If a word does not show up in the 
        /// `lines' it does not need necessarily be shown in the output.)</returns>
        /// <param name="lines">A collection of strings, containing each multiple words.</param>
        /// <param name="wordsToFind">A collection of strings, each a single word to find.</param>
        /// </summary>
        IEnumerable<Tuple<string, int>> Problem5(
            IEnumerable<string> lines,
            IEnumerable<string> wordsToFind);

        /// <summary>
        /// Generate the Cartesian product of two sets.
        /// </summary>
        /// <param name="leftSet">The first set of values.</param>
        /// <param name="rightSet">The second set of values.</param>
        /// <returns>The cartesian product of the left set and right set, as a collection of tuples.</returns>
        IEnumerable<Tuple<L, R>> Problem6<L, R>(
            IEnumerable<L> leftSet,
            IEnumerable<R> rightSet);

        /// <summary>
        /// Given a matrix of numbers, find the largest value on each row. (Matrix is a list of rows, each row is a list of values.).
        /// </summary>
        /// <param name="matrix">A matrix, given as a list of rows, each a list of values.</param>
        /// <returns>A collection of values, each being the largest in the row.</returns>
        IEnumerable<int> Problem7(IEnumerable<List<int>> matrix);

        /// <summary>
        /// <param name="matrix">A sparse matrix represented as triples (x,y,value)</param>. 
        /// <returns>The matrix transpose, in the same representation</returns> 
        /// </summary>
        IEnumerable<Tuple<int, int, double>> Problem8(
            IEnumerable<Tuple<int, int, double>> matrix);

        /// <summary>
        /// Add two sparse matrices.
        /// </summary>
        /// <param name="left">Left matrix to add.</param>
        /// <param name="right">Right matrix to add.</param>
        /// <returns>The sparse matrix that is the result.</returns>
        IEnumerable<Tuple<int, int, double>> Problem9(
            IEnumerable<Tuple<int, int, double>> left,
            IEnumerable<Tuple<int, int, double>> right);

        /// <summary>
        /// Multiply two sparse matrices.
        /// </summary>
        /// <param name="left">Left matrix to multiply.</param>
        /// <param name="right">Right matrix to multiply.</param>
        /// <returns>The sparse matrix that is the result.</returns>
        IEnumerable<Tuple<int, int, double>> Problem10(
            IEnumerable<Tuple<int, int, double>> left,
            IEnumerable<Tuple<int, int, double>> right);

        /// <summary>
        /// Consider a directed graph; node names are numbers. The graph is a list of edges 
        /// Tuple(int, int). Find all nodes reachable in exactly 5 steps starting from node 0.
        /// </summary>
        /// <param name="edges">List of edges in the graph.</param>
        /// <returns>All nodes reachable starting from node 0 in 5 steps.</returns>
        IEnumerable<int> Problem11(IEnumerable<Tuple<int, int>> edges);

        /// <summary>
        /// Compute two iteration steps of the pagerank of a directed graph.
        /// <param name="edges">List of edges in the graph.</param>
        /// </summary>
        /// <returns>For each node its weight after two iterations of pagerank.</returns>
        IEnumerable<Tuple<int, double>> Problem12(IEnumerable<Tuple<int, int>> edges);
    }

    public class Homework : IHomework
    {
        public static char[] wordDelimiters = new char[]
        {
            ' ', '\n', '\t', ',', '.', '!', '?', '\'', '\'', '`'
        };

        /// <summary>
        /// Use this function to split a string into words.
        /// </summary>
        /// <param name="s">String to split.</param>
        /// <returns>The words that are part of the string.</returns>
        public static IEnumerable<string> SplitStringIntoWords(string s)
        {
            return s.Split(Homework.wordDelimiters, StringSplitOptions.RemoveEmptyEntries);
        }

        /// <summary>
        /// Count the number of words in the input collection (words are separated by wordDelimiters).
        /// <param name="lines">A collection of strings, containing each multiple words.</param>
        /// </summary>
        /// <returns>The number of words in the input collection.</returns>
        public  int Problem1(IEnumerable<string> lines)
        {           
            throw new NotImplementedException();
        }

        /// <summary>
        /// Count the number of unique (distinct) words in the input collection.
        /// <param name="lines">A collection of strings, containing each multiple words.</param>
        /// <returns>The number of distinct (unique) words in the input collection.</returns>
        /// </summary>
        public  int Problem2(IEnumerable<string> lines)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Find the most frequent 10 words in the input collection, and their counts.
        /// <param name="lines">A collection of strings, containing each multiple words.</param>
        /// </summary>
        public  IEnumerable<Tuple<string, int>> Problem3(IEnumerable<string> lines)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// From each line of text extract just the vowels ('a', 'e', 'i', 'o', 'u').
        /// Example:
        /// Input: "How all occasions do inform against me",
        /// Output: Item1 => "How all occasions do inform against me",
        ///         Item2 => {o,a,o,a,i,o,o,i,o,a,a,i,e}
        /// <param name="lines">A collection of strings, containing each multiple words.</param>
        /// <returns>A list of lines and for each a list of vowels.</returns>
        /// </summary>
        public  IEnumerable<Tuple<string, IEnumerable<char>>> Problem4(IEnumerable<string> lines)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Given a list of words find the number of occurrences of each of them in the input collection.
        /// <returns>For each word the count.  (If a word does not show up in the 
        /// `lines' it does not need necessarily be shown in the output.)</returns>
        /// <param name="lines">A collection of strings, containing each multiple words.</param>
        /// <param name="wordsToFind">A collection of strings, each a single word to find.</param>
        /// </summary>
        public  IEnumerable<Tuple<string, int>> Problem5(
            IEnumerable<string> lines,
            IEnumerable<string> wordsToFind)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Generate the Cartesian product of two sets.
        /// </summary>
        /// <param name="leftSet">The first set of values.</param>
        /// <param name="rightSet">The second set of values.</param>
        /// <returns>The cartesian product of the left set and right set, as a collection of tuples.</returns>
        public  IEnumerable<Tuple<L, R>> Problem6<L, R>(
            IEnumerable<L> leftSet,
            IEnumerable<R> rightSet)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Given a matrix of numbers, find the largest value on each row. (Matrix is a list of rows, each row is a list of values.).
        /// </summary>
        /// <param name="matrix">A matrix, given as a list of rows, each a list of values.</param>
        /// <returns>A collection of values, each being the largest in the row.</returns>
        public  IEnumerable<int> Problem7(IEnumerable<List<int>> matrix)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// <param name="matrix">A sparse matrix represented as triples (x,y,value)</param>. 
        /// <returns>The matrix transpose, in the same representation</returns> 
        /// </summary>
        public  IEnumerable<Tuple<int, int, double>> Problem8(
            IEnumerable<Tuple<int, int, double>> matrix)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Add two sparse matrices.
        /// </summary>
        /// <param name="left">Left matrix to add.</param>
        /// <param name="right">Right matrix to add.</param>
        /// <returns>The sparse matrix that is the result.</returns>
        public  IEnumerable<Tuple<int, int, double>> Problem9(
            IEnumerable<Tuple<int, int, double>> left,
            IEnumerable<Tuple<int, int, double>> right)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Multiply two sparse matrices.
        /// </summary>
        /// <param name="left">Left matrix to multiply.</param>
        /// <param name="right">Right matrix to multiply.</param>
        /// <returns>The sparse matrix that is the result.</returns>
        public  IEnumerable<Tuple<int, int, double>> Problem10(
            IEnumerable<Tuple<int, int, double>> left,
            IEnumerable<Tuple<int, int, double>> right)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Consider a directed graph; node names are numbers. The graph is a list of edges 
        /// Tuple(int, int). Find all nodes reachable in exactly 5 steps starting from node 0.
        /// </summary>
        /// <param name="edges">List of edges in the graph.</param>
        /// <returns>All nodes reachable starting from node 0 in 5 steps.</returns>
        public  IEnumerable<int> Problem11(IEnumerable<Tuple<int, int>> edges)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Compute two iteration steps of the pagerank of a directed graph.
        /// <param name="edges">List of edges in the graph.</param>
        /// </summary>
        /// <returns>For each node its weight after two iterations of pagerank.</returns>
        public  IEnumerable<Tuple<int, double>> Problem12(IEnumerable<Tuple<int, int>> edges)
        {
            throw new NotImplementedException();
        }
    }
}