﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace TeachingLinq
{
    /// <summary>
    /// Sample input data.
    /// </summary>
    public interface IHomeworkInputData
    {
        /// <summary>
        /// A text containing a sequence of lines, each with multiple words.
        /// </summary>
        IEnumerable<string> InputText { get; }

        IEnumerable<string> WordsToFind { get; }

        IEnumerable<int> LeftSet { get; }

        IEnumerable<int> RightSet { get; }

        IEnumerable<List<int>> DenseMatrix { get; }

        IEnumerable<Tuple<int, int, double>> LeftSparseMatrix { get; }

        IEnumerable<Tuple<int, int, double>> RightSparseMatrix { get; }

        IEnumerable<Tuple<int, int>> GraphEdges { get; }
    }

    /// <summary>
    /// Example input data for testing.
    /// </summary>
    internal class SampleHomeworkInputData : IHomeworkInputData
    {
        private const string hamlet = @"
How all occasions do inform against me
And spur my dull revenge! What is a man,
If his chief good and market of his time
Be but to sleep and feed? A beast, no more.
Sure he that made us with such large discourse,
Looking before and after, gave us not
That capability and godlike reason
To fust in us unus'd. Now, whether it be
Bestial oblivion, or some craven scruple
Of thinking too precisely on th' event,- 
A thought which, quarter'd, hath but one part wisdom
And ever three parts coward,- I do not know
Why yet I live to say 'This thing's to do,'
Sith I have cause, and will, and strength, and means
To do't. Examples gross as earth exhort me. 
Witness this army of such mass and charge,
Led by a delicate and tender prince,
Whose spirit, with divine ambition puff'd,
Makes mouths at the invisible event,
Exposing what is mortal and unsure 
To all that fortune, death, and danger dare,
Even for an eggshell. Rightly to be great
Is not to stir without great argument,
But greatly to find quarrel in a straw
When honour's at the stake. How stand I then,
That have a father kill'd, a mother stain'd,
Excitements of my reason and my blood,
And let all sleep, while to my shame I see
The imminent death of twenty thousand men
That for a fantasy and trick of fame 
Go to their graves like beds, fight for a plot
Whereon the numbers cannot try the cause,
Which is not tomb enough and continent
To hide the slain? O, from this time forth,
My thoughts be bloody, or be nothing worth! 

    	     		Hamlet, Act 4, Scene 4";


        public IEnumerable<string> InputText
        {
            get { return hamlet.Split('\n').Select(l => l.Trim()); }
        }

        IEnumerable<string> wordsToFind = new string[]
            {
                "if", "and", "to", "of", "a", "my", "as", "not"
            };

        public IEnumerable<string> WordsToFind
        {
            get { return this.wordsToFind; }
        }

        public IEnumerable<int> LeftSet
        {
            get { return Enumerable.Range(0, 10); }
        }

        public IEnumerable<int> RightSet
        {
            get { return Enumerable.Range(20, 5); }
        }

        public IEnumerable<List<int>> DenseMatrix
        {
            get
            {
                return new List<List<int>>
                {
                    new List<int> {1, 2, 3, 4},
                    new List<int> {5, 6, 7, 8}
                };
                // This is the matrix 
                // [1 2 3 4]
                // [5 6 7 8] 
            }
        }

        public IEnumerable<Tuple<int, int, double>> LeftSparseMatrix 
        {
            get
            {
                return new List<Tuple<int, int, double>>
                {
                    Tuple.Create(0, 0, 1.0),
                    Tuple.Create(1, 1, 2.0),
                    Tuple.Create(0, 1, 3.0)
                };
                // This is the sparse matrix
                // [1 0]
                // [3 2]
            }
        }

        public IEnumerable<Tuple<int, int, double>> RightSparseMatrix
        {
            get { return this.LeftSparseMatrix; }
        }

        public IEnumerable<Tuple<int, int>> GraphEdges
        {
            get
            {
                return Enumerable.Range(0, 9)
                        .SelectMany(s =>
                                    Enumerable.Range(0, 9).Select(d => new Tuple<int, int>(s, d)))
                        .Where(e => (1 + 3 * e.Item1 + 5 * e.Item2) % 4 == 0).ToList();
            }
        }
    }

    class Program
    {
        public static void Show(object o)
        {
            DataDump dump = new DataDump(o);
            Console.WriteLine("{0}", dump.ToString());
        }

        public static void ShowResults(IHomework homework, IHomeworkInputData data)
        {
            Console.WriteLine("================Problem1");
            Program.Show(homework.Problem1(data.InputText));

            Console.WriteLine("================Problem2");
            Program.Show(homework.Problem2(data.InputText));

            Console.WriteLine("================Problem3");
            Program.Show(homework.Problem3(data.InputText));

            Console.WriteLine("================Problem4");
            Program.Show(homework.Problem4(data.InputText));

            Console.WriteLine("================Problem5");
            Program.Show(homework.Problem5(data.InputText, data.WordsToFind));

            Console.WriteLine("================Problem6");
            Program.Show(homework.Problem6<int, int>(data.LeftSet, data.RightSet));

            Console.WriteLine("================Problem7");
            Program.Show(homework.Problem7(data.DenseMatrix));

            Console.WriteLine("================Problem8");
            Program.Show(homework.Problem8(data.LeftSparseMatrix));

            Console.WriteLine("================Problem9");
            Program.Show(homework.Problem9(data.LeftSparseMatrix, data.RightSparseMatrix));

            Console.WriteLine("================Problem10");
            Program.Show(homework.Problem10(data.LeftSparseMatrix, data.RightSparseMatrix));

            Console.WriteLine("================Problem11");
            Program.Show(homework.Problem11(data.GraphEdges));

            Console.WriteLine("================Problem12");
            Program.Show(homework.Problem12(data.GraphEdges));
        }

        static void Main()
        {
#if !SHOWSLIDES
            Slides.ShowAllSlides();
#elif CHECKSOLUTIONS
            // This can only be done by teaching assistants
            IHomework solutions = new HomeworkSolutions();
            CompareSolutions.CheckStudentHomework(solutions);
#else // Display solutions
            IHomeworkInputData data = new SampleHomeworkInputData();
            IHomework solutions = new Homework();
            ShowResults(solutions, data);
#endif
            Console.ReadKey();
        }
    }
}
