﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace TeachingLinq
{
    internal class Slides
    {
        private static void Title(string title)
        {
            Console.WriteLine("==================");
            Console.WriteLine("Slide: {0}", title);
        }

        private static T Apply<T>(T argument, Func<T, T> operation)
        {
            return operation(argument);
        }

        public static void ShowAllSlides()
        {
            Title("IEnumerable<T>");
            IEnumerable<int> data = Enumerable.Range(0, 10);

            foreach (int n in data)
                Console.WriteLine(n);

            Program.Show(data);

            {
                Title("Using Anonymous Functions");
                Func<int, int> inc = x => x + 1;
                Console.WriteLine(inc(10));

                Title("Functions as First-Order Values");
                Console.WriteLine(Apply(10, inc));
            }

            {
                Title("Multiple arguments");
                Func<int, int, int> add = (x, y) => x + y;
                Console.WriteLine(add(3, 2));
            }

            {
                Title("Tuples");
                Tuple<int, int> pair = new Tuple<int, int>(5, 3);
                Tuple<int, string, double> triple = Tuple.Create(1, "hi", 3.14);
                Console.WriteLine("{0} {1} {2}", pair, triple, pair.Item1);
            }

            {
                Title("Select");
                var result = data.Select(d => d + 1);
                Program.Show(result);
            }

            {
                Title("Select can transform types");
                var result = data.Select(d => d > 5);
                Program.Show(result);
            }

            {
                Title("Chaining operators");
                var result = data
                    .Select(d => d * d)
                    .Select(d => Math.Sin(d))
                    .Select(d => d.ToString("0.00"));
                Program.Show(result);
            }

            {
                Title("Filtering with Where");
                var result = data.Where(d => d > 5);
                Program.Show(result);
            }

            {
                Title("Aggregations");
                var ct = data.Count();
                var sum = data.Sum();
                Console.WriteLine("{0} {1}", ct, sum);
            }

            {
                Title("General Aggregations");
                var sum = data.Aggregate((a, b) => a + b);
                Console.WriteLine("{0}", sum);
            }

            {
                Title("Aggregations with Seeds");
                var sum = data.Aggregate(
                    "X", // seed    
                    (a, b) => string.Format("({0} + {1})", a, b));
                Console.WriteLine("{0}", sum);
            }

            var lines = new string[] { "First string", "Second string" };

            {
                Title("Values can be collections");
                IEnumerable<int> counts = lines.Select(d => d.Count());
                Program.Show(counts);
            }

            {
                Title("SelectMany");
                IEnumerable<string> words = lines.SelectMany(d => d.Split(' '));
                Program.Show(words);

                IEnumerable<string[]> phrases = lines.Select(d => d.Split(' '));
                Program.Show(phrases);
            }

            {
                Title("GroupBy");
                IEnumerable<IGrouping<int, int>> groups = data.GroupBy(d => d % 3);
                Program.Show(groups);
            }

            {
                Title("IGrouping");
                var result = data.GroupBy(d => d % 3)
                                 .Select(g => g.Key);
                Program.Show(result);
            }

            {
                Title("Groups are Nested Collections");
                var result = data.GroupBy(d => d % 3)
                                 .Select(g => g.Count());
                Program.Show(result);
            }

            {
                Title("Sorting");
                var sorted = lines.SelectMany(l => l.Split(' '))
                                  .OrderBy(l => l.Length);

                Program.Show(sorted);
            }

            {
                Title("Joins");
                var L = Enumerable.Range(0, 5);
                var R = Enumerable.Range(0, 4);
                var result = L.Join(R, l => l % 2, r => (r + 1) % 2, (l, r) => l + r);
                Program.Show(result);
            }

            {
                Title("Join Example");
                var words = lines.SelectMany(d => d.Split(' '));
                var common = words.Join(data, s => s.Length, d => d,
                                        (s, d) => new { length = d, str = s });

                Program.Show(common);
            }

            {
                Title("Take");
                var first5 = data.Select(d => d * d).Take(5);
                Program.Show(first5);
            }
        }
    }
}